/* 
 * Copyright (C) 2004 Volker Stroebel <volker@planetpenguin.de>
 * 
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "button.h"

#include "textures.h"
#include "fonts.h"

#include "ui_mgr.h"


namespace pp {
	
Button::Button( pp::Vec2d pos, pp::Vec2d size,
			 char *font_binding, const char *label )
{
    this->position = pos;
    this->size = size;

    this->tex.binding = NULL;
    this->hilit_tex.binding = NULL;
    this->clicked_tex.binding = NULL;
    this->disabled_tex.binding = NULL;
    this->font_binding = font_binding;
    this->disabled_font_binding = NULL;
    this->hilit_font_binding = NULL;
    this->label = label;

    this->clicked = false;
    this->focused = false;
    this->enabled = true;
    this->visible = false;
    this->active = false;
}

void
Button::setHilitFontBinding( char *binding )
{
   hilit_font_binding = binding;
}

void
Button::setImage( char *texture_binding,
		       pp::Vec2d p0, pp::Vec2d p1, pp::Color color )
{
    tex.binding = texture_binding;
    tex.ll = p0;
    tex.ur = p1;
    tex.color = color;
}

void
Button::setDisabledImage(char *texture_binding,
				pp::Vec2d p0, pp::Vec2d p1, pp::Color color )
{
	disabled_tex.binding = texture_binding;
    disabled_tex.ll = p0;
    disabled_tex.ur = p1;
    disabled_tex.color = color;
}

void
Button::setHighlight(bool highlight)
{
    if (!(focused == highlight)) {
		UIMgr.setDirty();
    }
    focused = highlight;
}

void
Button::setHilitImage(char *texture_binding,
			     pp::Vec2d p0, pp::Vec2d p1, pp::Color color )
{
    hilit_tex.binding = texture_binding;
    hilit_tex.ll = p0;
    hilit_tex.ur = p1;
    hilit_tex.color = color;
}

void
Button::setClickedImage(char *texture_binding,
			       pp::Vec2d p0, pp::Vec2d p1, pp::Color color )
{
    clicked_tex.binding = texture_binding;
    clicked_tex.ll = p0;
    clicked_tex.ur = p1;
    clicked_tex.color = color;
}

void
Button::setDisabledFontBinding( char *binding )
{
	disabled_font_binding = binding;
}

void
Button::draw()
{
    GLuint texobj;
    texture_region_t *tex;
    pp::Vec2d pos;
    double w, h;
    char *font_binding;

    check_assertion( button != NULL, "button is NULL" );

    pos = this->position;
    w = size.x;
    h = size.y;

    glEnable( GL_TEXTURE_2D );

    tex = NULL;
    font_binding = NULL;

    if ( !this->enabled ) {
	if ( this->disabled_tex.binding ) {
	    tex = &this->disabled_tex;
	} else if ( this->tex.binding ) {
	    tex = &this->tex;
	}

	if ( this->disabled_font_binding ) {
	    font_binding = this->disabled_font_binding;
	} else if ( this->font_binding ) {
	    font_binding = this->font_binding;
	}
    } else if ( this->clicked ) {
	if ( this->clicked_tex.binding ) {
	    tex = &this->clicked_tex;
	} else if ( this->hilit_tex.binding ) {
	    tex = &this->hilit_tex;
	} else if ( this->tex.binding ) {
	    tex = &this->tex;
	} 

	if ( this->hilit_font_binding ) {
	    font_binding = this->hilit_font_binding;
	} else if ( this->font_binding ) {
	    font_binding = this->font_binding;
	}
    } else if ( this->focused ) {
	if ( this->hilit_tex.binding ) {
	    tex = &this->hilit_tex;
	} else if ( this->tex.binding ) {
	    tex = &this->tex;
	} 

	if ( this->hilit_font_binding ) {
	    font_binding = this->hilit_font_binding;
	} else if ( this->font_binding ) {
	    font_binding = this->font_binding;
	}
    } else {
	if ( this->tex.binding ) {
	    tex = &this->tex;
	} 

	if ( this->font_binding ) {
	    font_binding = this->font_binding;
	}
    }

    if ( tex != NULL ) {
	if ( !get_texture_binding( tex->binding, &texobj ) ) {
	    print_warning( IMPORTANT_WARNING,
			   "Couldnt get texture object for binding %s",
			   tex->binding );
	    texobj = 0;
	}

	glBindTexture( GL_TEXTURE_2D, texobj );

	glColor4dv( (double*) &tex->color );

	glBegin( GL_QUADS );
	{
	    glTexCoord2f( tex->ll.x, tex->ll.y );
	    glVertex3f( pos.x, pos.y, 0 );

	    glTexCoord2f( tex->ur.x, tex->ll.y );
	    glVertex3f( pos.x + w, pos.y, 0 );

	    glTexCoord2f( tex->ur.x, tex->ur.y );
	    glVertex3f( pos.x + w, pos.y + h, 0 );

	    glTexCoord2f( tex->ll.x, tex->ur.y );
	    glVertex3f( pos.x, pos.y + h, 0 );
	}
	glEnd();
    }

    if ( font_binding && this->label != NULL ) {
	font_t font;
	int w, asc, desc;

	if (!get_font_binding( font_binding, font )) {
	    print_warning( IMPORTANT_WARNING, 
			   "Couldn't get font object for binding %s",
			   font_binding );
	    //font = NULL;
	} else {
	    bind_font_texture( &font );

	    get_font_metrics( &font, this->label, &w, &asc, &desc );

	    glPushMatrix();
	    {
		glTranslatef( this->position.x + this->size.x/2.0 - w/2.0,
			      this->position.y + this->size.y/2.0 - asc/2.0 + desc/2.0,
			      0.0 );

		draw_string( &font, this->label );
	    
	    }
	    glPopMatrix();
	}
    }
    
}

} //namespace pp
